#
# pygamepal, by Rik Cross
#  -- homepage: github.com/rik-cross/pygamepal
#  -- MIT licenced, free to use, modify and distribute
#  -- run 'pip install pygamepal' to use
#
# instructions:
#  -- arrow keys to pan
#  -- z/x to zoom
#
# Image credit - Cup Nooble
#  --  cupnooble.itch.io/sprout-lands-asset-pack
#

import pygame
import pygamepal
import os

# initialise Pygame
pygame.init()

# setup screen to required size
screen = pygame.display.set_mode((680, 460))
pygame.display.set_caption('Camera Example')
clock = pygame.time.Clock() 

# add input, to allow camera control
input = pygamepal.Input()

# load a texture
texture = pygame.image.load(os.path.join('images','character.png'))

# create surface for the camera to draw
cameraSurface = pygame.Surface((200, 200), pygame.SRCALPHA, 32)

camera = pygamepal.Camera(position=(300, 100),
                        size=(300, 300),
                        zoom=5,
                        # the camera center
                        target=(0, 0),
                        borderThickness=4,
                        backgroundColour='gray10',
                        # lazy follow
                        followDelay=0.9)

input = pygamepal.Input()

# game loop
running = True
while running:

    # clear screen to Cornflower Blue
    screen.fill('cornflowerblue')

    # advance clock at 60 FPS
    clock.tick(60)

    # respond to quit event
    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            running = False

    #
    # input
    #

    # arrow keys to pan
    if input.isKeyDown(pygame.K_LEFT):
        camera.target = (camera.target[0]-1, camera.target[1])
    if input.isKeyDown(pygame.K_RIGHT):
        camera.target = (camera.target[0]+1, camera.target[1])
    if input.isKeyDown(pygame.K_UP):
        camera.target = (camera.target[0], camera.target[1]-1)
    if input.isKeyDown(pygame.K_DOWN):
        camera.target = (camera.target[0], camera.target[1]+1)
    # z/x to zoom
    if input.isKeyDown(pygame.K_z):
        camera.zoom -= 0.1
    if input.isKeyDown(pygame.K_x):
        camera.zoom += 0.1

    #
    # update
    #

    input.update()
    camera.update()

    #
    # draw
    #

    # don't forget to clear the camera surface!
    cameraSurface.fill((0, 0, 0, 0))

    # draw multiple images to the surface to be rendered by the camera
    for i in range(0, 200, 25):
        for j in range(0, 200, 25):
                cameraSurface.blit(texture, (i, j))

    # draw the images (without the camera) 
    screen.blit(cameraSurface, (0, 0))

    # draw the instructions
    pygamepal.drawText(screen, 'Arrow keys to pan, z/x to zoom', 300, 60)

    # use the camera to draw the images on the cameraSurface to the screen
    camera.draw(cameraSurface, screen)

    # draw to the screen
    pygame.display.flip()

# quit Pygame on exit
pygame.quit()



#
# pygamepal, by Rik Cross
#  -- homepage: github.com/rik-cross/pygamepal
#  -- MIT licenced, free to use, modify and distribute
#  -- run 'pip install pygamepal' to use
#

import pygame
import pygamepal

#
# create a new game
#

class MyGame(pygamepal.Game):

    def init(self):
        # replace 'pass' below with your code
        pass

    def update(self):
        # replace 'pass' below with your code
        pass

    def draw(self):
        # replace the code below with your code
        self.screen.fill('cornflowerblue')

#
# create a new game instance
#

myGame = MyGame()
myGame.run()


#
# pygamepal, by Rik Cross
#  -- homepage: github.com/rik-cross/pygamepal
#  -- MIT licenced, free to use, modify and distribute
#  -- run 'pip install pygamepal' to use
#
# Instructions
#  -- arrow keys to move player
#
# Image credit - Cup Nooble
#  --  cupnooble.itch.io/sprout-lands-asset-pack
#

import pygame
import pygamepal
import os

#
# load spritesheet and split into separate images
#

# load a texture
texture = pygame.image.load(os.path.join('images','character_spritesheet.png'))
# double the texture size
texture = pygame.transform.scale(texture, (texture.get_width()*2,texture.get_height()*2))
# split texture into a 2D list of sub-textures
splitTextures = pygamepal.splitTexture(texture, 96, 96)

#
# chest (just an image)
#

chestImage = pygame.image.load(os.path.join('images', 'chest.png'))
chestPosition = (200, 200)

#
# create a new game
#

class MyGame(pygamepal.Game):

    def init(self):

        # set window properties
        self.size = (800, 500)
        self.caption = 'Full game example'
        
        # create an input object
        self.input = pygamepal.Input()

        #
        # create a player sprite
        #

        self.player = pygame.sprite.Sprite()
        
        # add a spriteImage to the player sprite, including 5 different states
        self.player.spriteImage = pygamepal.SpriteImage()
        self.player.spriteImage.addTextures(splitTextures[0][0], splitTextures[0][1], state='idle', offset=(17*2, 16*2))
        self.player.spriteImage.addTextures(splitTextures[0][1], splitTextures[0][2], splitTextures[0][1], splitTextures[0][3], state='walk_down', offset=(17*2, 16*2))
        self.player.spriteImage.addTextures(splitTextures[1][1], splitTextures[1][2], splitTextures[1][1], splitTextures[1][3], state='walk_up', offset=(17*2, 16*2))
        self.player.spriteImage.addTextures(splitTextures[2][1], splitTextures[2][2], splitTextures[2][1], splitTextures[2][3], state='walk_left', offset=(17*2, 16*2))
        self.player.spriteImage.addTextures(splitTextures[3][1], splitTextures[3][2], splitTextures[3][1], splitTextures[3][3], state='walk_right', offset=(17*2, 16*2))
        
        # set player position and size
        #self.player.position = [160, 160]
        #self.player.size = (28, 32)
        self.player.rect = pygame.Rect(160, 160, 28, 32)

        #
        # create a camera object
        #

        self.camera = pygamepal.Camera(position=(50, 50),
                                          size=(700, 400),
                                          # center the camera on the player
                                          target=(self.player.rect.x + self.player.rect.w/2, 
                                                  self.player.rect.y + self.player.rect.h/2),
                                          zoom=3,
                                          backgroundColour='darkgreen',
                                          # clamp the camera to the world
                                          clamp=True,
                                          clampRect=(0, 0, 500, 500),
                                          followDelay=0.9)
        
        # create a separate surface for drawing the world
        # this surface will be used by the camera
        self.worldSurface = pygame.surface.Surface((500, 500), pygame.SRCALPHA, 32)
        

    def update(self):

        # fill the screen
        self.screen.fill('cornflowerblue')

        # update the input
        self.input.update()

        # arrow keys to:
        # - change state of player sprite
        # - change player sprite position (clamped to world)
        if self.input.isKeyDown(pygame.K_UP):
            self.player.spriteImage.setState('walk_up')
            self.player.rect.y = max(0, self.player.rect.y - 1)
        elif self.input.isKeyDown(pygame.K_DOWN):
            self.player.spriteImage.setState('walk_down')
            self.player.rect.y = min (500 - self.player.rect.h, self.player.rect.y + 1)
        elif self.input.isKeyDown(pygame.K_LEFT):
            self.player.spriteImage.setState('walk_left')
            self.player.rect.x = max(0, self.player.rect.x - 1)
        elif self.input.isKeyDown(pygame.K_RIGHT):
            self.player.spriteImage.setState('walk_right')
            self.player.rect.x = min(500 - self.player.rect.w, self.player.rect.x + 1)
        # idle state is the default
        else:
            self.player.spriteImage.setState('idle')

        # update the sprite image
        self.player.spriteImage.update()
        
        # update the camera
        self.camera.update()
        
        # track the player with the camera
        self.camera.target = (self.player.rect.x + self.player.rect.w/2, 
                              self.player.rect.y + self.player.rect.h/2)
        
    def draw(self):

        # clear the world surface
        self.worldSurface.fill((0, 0, 0, 0))
        # draw the player sprite image to the world surface
        self.player.spriteImage.draw(self.worldSurface, self.player.rect.x, self.player.rect.y)
        # draw the chest to the world surface
        self.worldSurface.blit(chestImage, chestPosition)
        # use the camera to draw the world surface to the screen
        self.camera.draw(self.worldSurface, self.screen)

#
# create a new game instance
#

myGame = MyGame()
myGame.run()


#
# pygamepal, by Rik Cross
#  -- homepage: github.com/rik-cross/pygamepal
#  -- MIT licenced, free to use, modify and distribute
#  -- run 'pip install pygamepal' to use
#

import pygame

#
# game abstraction
#

class Game:

    def __init__(self):
        pygame.init()
        self.size = (640, 480)
        self.caption = ''
        self.fps = 60
        self.init()
        self.screen = pygame.display.set_mode(self.size)
        pygame.display.set_caption(self.caption)
        self.clock = pygame.time.Clock() 
        
        self.running = False

    def init(self):
        pass

    def _update(self):
        # respond to quit event
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                self.running = False
        # run user update method
        self.update()
        # update clock
        self.clock.tick(self.fps)

    def update(self):
        pass

    def _draw(self):
        self.draw()
        # present the screen
        pygame.display.flip()

    def draw(self):
        pass

    def run(self):
        self.running = True
        while self.running:
            self._update()
            self._draw()
        pygame.quit()

    @property
    def icon(self):
        return self._icon
    
    @icon.setter
    def icon(self, value):
        self._icon = value
        pygame.display.set_icon(self._icon)

#
# input manager
#

class Input():

    # longPressDuraton: the number of milliseconds/frames a key needs to be held to register a long press
    def __init__(self, longPressDuration=60):
        self.longPressDuration = longPressDuration
        # set key states
        self.currentKeyStates = pygame.key.get_pressed()
        self.previousKeyStates = pygame.key.get_pressed()
        # set long press durations
        self._durations = [0 for _ in range(len(self.currentKeyStates))]

    def update(self, deltaTime=1):
        # update key presses
        self.previousKeyStates = self.currentKeyStates
        self.currentKeyStates = pygame.key.get_pressed()
        # update press durations
        for i in range(len(self._durations)):
            if self.currentKeyStates[i]:
                self._durations[i] += deltaTime
            else:
                self._durations[i] = 0

    # key methods

    # returns true if the key denoted by keyCode
    # is held down during the current frame
    def isKeyDown(self, keyCode):
        if self.currentKeyStates is None or self.previousKeyStates is None:
            return False
        return self.currentKeyStates[keyCode] == True

    # returns true if the key denoted by keyCode
    # has been pressed down during the current frame
    def isKeyPressed(self, keyCode):
        if self.currentKeyStates is None or self.previousKeyStates is None:
            return False
        return self.currentKeyStates[keyCode] == True and self.previousKeyStates[keyCode] == False

    # returns true if the key denoted by keyCode
    # has been released during the current frame
    def isKeyReleased(self, keyCode):
        if self.currentKeyStates is None or self.previousKeyStates is None:
            return False
        return self.currentKeyStates[keyCode] == False and self.previousKeyStates[keyCode] == True

    # long key methods

    # returns the number of frames a keyCode has been held down
    def getKeyDownDuration(self, keyCode):
        return self._durations[keyCode]

    # returns true if the key denoted by keyCode
    # is held down for a long press during the current frame
    def isKeyLongDown(self, keyCode):
        return self._durations[keyCode] >= self.longPressDuration

    # returns true if the key denoted by keyCode
    # has achieved a long press in the current frame
    def isKeyLongPressed(self, keyCode):
        return self._durations[keyCode] == self.longPressDuration

    # returns keyCode progress towards a long press (0%-100%)
    def GetKeyLongPressPercentage(self, keyCode):
        return min(100, self._durations[keyCode] / self.longPressDuration * 100)

#
# Sprites and animations
#

# returns an array of textures, split from the passed texture (spritesheet)
# (this could be a 2-dimensional array)

def splitTexture(texture, newTextureWidth, newTextureHeight):

    # the list of textures to return
    newTextures = []

    # use texture.subsurface to loop through the image
    for row in range(0, texture.get_height(), newTextureHeight):
        newRow = []
        for column in range(0, texture.get_width(), newTextureWidth):
            # add the cropped texture to the list
            newRow.append(texture.subsurface(
                column, row, newTextureWidth, newTextureHeight))
        newTextures.append(newRow)

    return newTextures

# flatten a 2D or [n]D list into a single list

def flatten(list):
    newList = []
    for i in list:
        for j in i:
            newList.append(j)
    return newList

# used for storing a list of texture (and other info)
# for a spriteImage state

class TextureList():
    def __init__(self):
        self._textures = []
        self._animationDelay = 12
        self._loop = True
        self._hFlip = False
        self._vFlip = False
        self._offset = (0, 0)

# allows for one or more tetxures to be associated with a state

class SpriteImage():

    def __init__(self):
        self._reset()

    # set object to initial values
    def _reset(self):
        # maps a state to a list of textures
        self._textureLists = {}
        self._currentState = None
        # keeps track of current animation frame and progress
        self._animationIndex = 0
        self._animationTimer = 0
        self.pause = False

    # add one or more textures, with an associated state
    def addTextures(self, firstTexture, *moreTextures, state=None, animationDelay=12, loop=True, hFlip=False, vFlip=False, offset=(0,0)):
        # allow textures with no attached state (for single-state images/animations)
        if state is None:
            state = 'default'
        # get existing textureList or create a new one
        if state not in self._textureLists:
            textureList = TextureList()
        else:
            textureList = self._sprites[state]
        # add textures to list
        textureList._textures.append(firstTexture)
        for texture in moreTextures:
            textureList._textures.append(texture)
        # add attributes for the current state
        textureList._animationDelay = animationDelay
        textureList._loop = loop
        textureList._hFlip = hFlip
        textureList._vFlip = vFlip
        textureList._offset = offset
        self._textureLists[state] = textureList
        if len(self._textureLists) == 1:
            self._currentState = state

    def getCenter(self):
        if self._currentState is None or self.pause:
            return (0, 0)
        # get the current animation frame
        currentTexture = self._textureLists[self._currentState]._textures[self._animationIndex]
        return (currentTexture.get_width()/2, currentTexture.get_height()/2)

    # must be called once per frame to update sprite
    def update(self):
        # only animate if there's at least one image and it's not paused
        if self._currentState is None or self.pause:
            return
        # increment timer
        self._animationTimer += 1
        # advance animation if timer reaches delay value
        if self._animationTimer >= self._textureLists[self._currentState]._animationDelay:
            self._animationTimer = 0
            self._animationIndex += 1
            if self._animationIndex > len(self._textureLists[self._currentState]._textures) - 1:
                if self._textureLists[self._currentState]._loop:
                    self._animationIndex = 0
                else:
                    self._animationIndex = len(
                        self._textureLists[self._currentState]._textures) - 1

    def setState(self, state):
        # only change state if the new state is different
        if self._currentState == state:
            return
        # set the new state
        self._currentState = state
        # reset index and timer for the new state
        self.resetState()

    def draw(self, screen, x, y):
        # don't draw if there's nothing to draw
        if self._currentState is None or self._textureLists[self._currentState] == None:
            return
        # get the current animation frame
        currentTexture = self._textureLists[self._currentState]._textures[self._animationIndex]
        # draw (optionally flipped) texture
        screen.blit(pygame.transform.flip(currentTexture,
                                          self._textureLists[self._currentState]._hFlip,
                                          self._textureLists[self._currentState]._vFlip),
                    (x - self._textureLists[self._currentState]._offset[0],
                     y - self._textureLists[self._currentState]._offset[1],
                     currentTexture.get_width(),
                     currentTexture.get_height()))
    # puts sprite animation back to start

    def resetState(self):
        self._animationIndex = 0
        self._animationTimer = 0

#
# Camera
#

class Camera:

    def __init__(self, position=(0, 0), size=(640, 480),
                 target=(0, 0), zoom=1, backgroundColour='gray30',
                 borderColour='black', borderThickness=2, 
                 clamp=False, clampRect=(0, 0, 1000, 1000),
                 followDelay=0):
        self.position = position
        self.size = size
        # sets the camera center
        self.target = target
        self._currentTarget = self.target
        self._zoom = zoom
        self.backgroundColour = backgroundColour
        self.borderColour = borderColour
        self.borderThickness = borderThickness
        self.clamp = clamp
        self.clampRect = clampRect
        # follow delay: value between 0 (instant snap to target) and 1 (no movement)
        self.followDelay = followDelay

    def update(self, deltaTime=1):
        # update the target using the follow amount
        self._currentTarget = (self._currentTarget[0]*self._followDelay + self.target[0]*(1-self._followDelay),
                         self._currentTarget[1]*self._followDelay + self.target[1]*(1-self._followDelay))

    # draws the surface to the destination surface
    # using the camera attributes
    def draw(self, surface, destSurface):
        # draw border
        pygame.draw.rect(destSurface, self.borderColour, 
                         (self.position[0] - self.borderThickness, self.position[1] - self.borderThickness, 
                          self.size[0] + self.borderThickness*2, self.size[1] + self.borderThickness*2))
        # ensure that the surface is clipped to the camera dimensions
        destSurface.set_clip((self.position[0], self.position[1], self.size[0], self.size[1]))
        # fill the surface to the background colour
        destSurface.fill(self.backgroundColour)
        # blit the (zoomed) surface to the destination, and set the target as the center
        x = 0 - (self.size[0]/2 - self._currentTarget[0] * self._zoom)
        y = 0 - (self.size[1]/2 - self._currentTarget[1] * self._zoom)
        # clamp to clampRect coordinates
        if self.clamp:
            x = max(self.clampRect[0] * self.zoom, min(self.clampRect[2] * self.zoom - self.size[0], x))
            y = max(self.clampRect[1] * self.zoom, min(self.clampRect[3] * self.zoom - self.size[1], y))
        # draw the surface to the destination using the correct position, size, center and zoom
        destSurface.blit(pygame.transform.scale(surface, (surface.get_width()*self._zoom, surface.get_height()*self._zoom)), 
                         self.position, 
                         (x, y,
                          self.size[0], self.size[1]))
        # reset surface clipping
        destSurface.set_clip()

    # zoom property, to avoid zoom of < 0.1
    @property
    def zoom(self):
        return self._zoom
    
    @zoom.setter
    def zoom(self, value):
        self._zoom = max(value, 0.1)

    # followAmount property, to clamp value between 0 and 1
    @property
    def followDelay(self):
        return self._followDelay
    
    @followDelay.setter
    def followDelay(self, value):
        self._followDelay = max(0, min(1, value))

#
# draw utils
#

# easily draw text

def drawText(screen, text, x, y, font=None, antialias=True, colour=(255, 255, 255), background=None):
    # use 'standard' font if none specified
    if font is None:
        font = pygame.font.SysFont(None, 24)
    # create text surface
    textSurface = font.render(text, antialias, colour, background)
    # draw text to screen
    screen.blit(textSurface, (x, y))



#
# pygamepal, by Rik Cross
#  -- homepage: github.com/rik-cross/pygamepal
#  -- MIT licenced, free to use, modify and distribute
#  -- run 'pip install pygamepal' to use
#
# Instructions
#  -- press the 'return' key and watch console output
#

# import modules
import pygame
import pygamepal

# initialise Pygame
pygame.init()

# setup screen
screen = pygame.display.set_mode((680, 460))
pygame.display.set_caption('Input Example')
clock = pygame.time.Clock()

input = pygamepal.Input()

# game loop
running = True
while running:

    # clear screen
    screen.fill('cornflowerblue')

    # advance clock (10 FPS so that text is easier to read)
    clock.tick(10)

    deltaTime = clock.get_time()

    # respond to quit event
    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            running = False

    #
    # input
    #

    # none

    #
    # update
    #

    # update input manager
    input.update(deltaTime)

    #
    # draw
    #

    # input text
    key = pygame.K_RETURN
    text = "Return key -- down: " + str(input.isKeyDown(key)) + \
        ", pressed: " + str(input.isKeyPressed(key)) + \
        ", released: " + str(input.isKeyReleased(key)) + \
        ", duration: " + str(input.getKeyDownDuration(key)) + \
        ", long down: " + str(input.isKeyLongDown(key)) + \
        ", long press: " + str(input.isKeyLongPressed(key)) + \
        ", long%: " + \
        "{:.2f}".format(input.GetKeyLongPressPercentage(key))

    print(text)

    # draw to screen
    pygame.display.flip()

#  quit Pygame
pygame.quit()



#
# pygamepal, by Rik Cross
#  -- homepage: github.com/rik-cross/pygamepal
#  -- MIT licenced, free to use, modify and distribute
#  -- run 'pip install pygamepal' to use
#
# instructions:
#  -- space to toggle pause sprite 2
#  -- arrow keys to control sprite 3
#
# Image credit - Cup Nooble
#  --  cupnooble.itch.io/sprout-lands-asset-pack
#

# import modules
import pygame
import pygamepal
import os

# initialise Pygame
pygame.init()

# setup screen
screen = pygame.display.set_mode((680, 460))
pygame.display.set_caption('SpriteImage Example')
clock = pygame.time.Clock() 

# add input, to easily query keys
input = pygamepal.Input()

# load a texture
texture = pygame.image.load(os.path.join('images','character_spritesheet.png'))
# double the texture size
texture = pygame.transform.scale(texture, (texture.get_width()*2,texture.get_height()*2))
# split texture into a 2D list of sub-textures
splitTextures = pygamepal.splitTexture(texture, 96, 96)

# a sprite with a single texture
spriteImage1 = pygamepal.SpriteImage()
# simple alternative for single texture: spriteImage1.addTextures(pygame.image.load('image.png'))
spriteImage1.addTextures(splitTextures[0][0], offset=(17*2, 16*2))

# an animated sprite with multiple textures
spriteImage2 = pygamepal.SpriteImage()
spriteImage2.addTextures(splitTextures[3][1], splitTextures[3][2], splitTextures[3][1], splitTextures[3][3], offset=(17*2, 16*2))
# simple alternative for single textures:
# spriteImage1.addTextures(pygame.image.load('image1.png'), pygame.image.load('image2.png'))

# a controllable sprite with multiple animation states
spriteImage3 = pygamepal.SpriteImage()
spriteImage3.addTextures(splitTextures[0][0], splitTextures[0][1], state='idle', offset=(17*2, 16*2))
spriteImage3.addTextures(splitTextures[0][1], splitTextures[0][2], splitTextures[0][1], splitTextures[0][3], state='walk_down', offset=(17*2, 16*2))
spriteImage3.addTextures(splitTextures[1][1], splitTextures[1][2], splitTextures[1][1], splitTextures[1][3], state='walk_up', offset=(17*2, 16*2))
spriteImage3.addTextures(splitTextures[2][1], splitTextures[2][2], splitTextures[2][1], splitTextures[2][3], state='walk_left', offset=(17*2, 16*2))
spriteImage3.addTextures(splitTextures[3][1], splitTextures[3][2], splitTextures[3][1], splitTextures[3][3], state='walk_right', offset=(17*2, 16*2))

# for easily getting key presses
input = pygamepal.Input()

# game loop
running = True
while running:

    # clear screen
    screen.fill('cornflowerblue')
    # advance clock
    clock.tick(60)

    # respond to quit event
    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            running = False

    #
    # input
    #

    # space to toggle pause s2 sprite
    if input.isKeyPressed(pygame.K_SPACE):
        spriteImage2.pause = not spriteImage2.pause

    # arrow keys to change state of s3 sprite
    if input.isKeyDown(pygame.K_UP):
        spriteImage3.setState('walk_up')
    elif input.isKeyDown(pygame.K_DOWN):
        spriteImage3.setState('walk_down')
    elif input.isKeyDown(pygame.K_LEFT):
        spriteImage3.setState('walk_left')
    elif input.isKeyDown(pygame.K_RIGHT):
        spriteImage3.setState('walk_right')
    # idle state is the default
    else:
        spriteImage3.setState('idle')

    #
    # update
    #

    input.update()
    spriteImage1.update()
    spriteImage2.update()
    spriteImage3.update()

    #
    # draw
    #

    # draw split texture
    for col in range(len(splitTextures)):
        for row in range(len(splitTextures[0])):
            pygame.draw.rect(screen, 'gray10', (col * 100, row * 100, 96, 96), False)
            screen.blit(splitTextures[row][col], (col * 100, row * 100, 96, 96))

    # draw sprites and accompanying text
    pygamepal.drawText(screen, 'Sprite 1 (single texture)', 420, 40)
    spriteImage1.draw(screen, 500, 70)
    pygamepal.drawText(screen, 'Sprite 2 (space to pause/play)', 420, 140)
    spriteImage2.draw(screen, 500, 170)
    pygamepal.drawText(screen, 'Sprite 3 (arrow keys to change)', 420, 240)
    spriteImage3.draw(screen, 500, 270)

    # draw to screen
    pygame.display.flip()

# quit Pygame
pygame.quit()



#
# pygamepal, by Rik Cross
#  -- homepage: github.com/rik-cross/pygamepal
#  -- MIT licenced, free to use, modify and distribute
#  -- run 'pip install pygamepal' to use
#
# Image credit - Cup Nooble
#  --  cupnooble.itch.io/sprout-lands-asset-pack
#

import pygame
import pygamepal
import os

#
# create a new game
#

class MyGame(pygamepal.Game):

    def init(self):
        # this code is optional and can be removed or replaced
        self.size = (640, 480)
        self.caption = 'Example Game'
        self.icon = pygame.image.load(os.path.join('images', 'character.png'))
        self.blue = 0

    def update(self):
        # this code is optional and can be removed or replaced
        # cycle a value between 0 and 255
        self.blue = (self.blue + 1) % 255

    def draw(self):
        # this code is optional and can be removed or replaced
        self.screen.fill('cornflowerblue')
        pygamepal.drawText(self.screen, 'pygamepal example game!', 25, 25, colour=(0, 0, self.blue))
        pygamepal.drawText(self.screen, 'Add code to init(), update() and draw() methods.', 25, 50)

#
# create a new game instance
#

myGame = MyGame()
myGame.run()